package com.caimei365.tools.controller;

import com.caimei365.tools.model.ResponseJson;
import com.caimei365.tools.model.dto.MessageDto;
import com.caimei365.tools.service.RocketMqService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.apache.rocketmq.client.producer.SendResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/6/17
 */
@Api(tags="MQ消息中间件API")
@RestController
@RequiredArgsConstructor
@RequestMapping("/tools")
public class RocketMqApi {

    private final RocketMqService rocketMqService;

    /**
     * 发送消息
     *
     * @param messageDto {
     *                    topic    消息主题
     *                    content  消息内容
     *                    tag      消息标签(可选)
     *                    sort     有序消息(可选)：1是，0否
     *                    async    异步消息(可选)：1是，0否
     *                    oneway   单向消息(可选)：1是，0否
     *                    delay    延时消息等级(可选)：1-18，0否，对应时间依次：1s 5s 10s 30s 1m 2m 3m 4m 5m 6m 7m 8m 9m 10m 20m 30m 1h 2h
     *                  }
     Posteturn SendResult
     */
    @ApiOperation("发送消息")
    @PostMapping("/mq/send")
    public ResponseJson<SendResult> sendCommonMessage(MessageDto messageDto) {
        if (StringUtils.isEmpty(messageDto.getTopic())) {
            return ResponseJson.error("消息主题不能为空！", null);
        }
        if (StringUtils.isEmpty(messageDto.getContent())) {
            return ResponseJson.error("消息内容不能为空！", null);
        }
        SendResult sendResult = rocketMqService.sendCommonMessage(messageDto);
        return ResponseJson.success(sendResult);
    }


}
