package com.caimei365.tools.controller;

import com.alibaba.fastjson.JSONObject;
import com.caimei365.tools.model.ResponseJson;
import com.caimei365.tools.model.dto.SmsDto;
import com.caimei365.tools.utils.SmsUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/6/18
 */
@Slf4j
@Api(tags="发送短信API")
@RestController
@RequiredArgsConstructor
@RequestMapping("/tools")
public class SmsApi {

    /**
     * 发送短信
     *
     * @param smsDto {
     *                type    短信类型：1通知短信，2验证码短信，3营销短信
     *                mobile  手机号
     *                content 短信内容
     *               }
     */
    @ApiOperation("发送消息")
    @PostMapping("/sms/send")
    public ResponseJson<String> sendSms(SmsDto smsDto){
        String result = SmsUtil.sendSms(smsDto.getType(), smsDto.getMobile(), smsDto.getContent());
        JSONObject json = (JSONObject) JSONObject.parse(result);
        log.info("发送结果：" + result + "，短信内容：" + smsDto.toString());
        if (null != json && json.getInteger("code") == 0) {
            return ResponseJson.success(0, "发送成功",null);
        } else {
            return ResponseJson.error(0, "发送失败", result);
        }
    }

}
