package com.caimei365.tools.controller;

import com.caimei365.tools.model.ResponseJson;
import com.caimei365.tools.service.UploadService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpHeaders;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.util.Map;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/10/25
 */
@Slf4j
@Api(tags="文件上传API")
@RestController
@RequiredArgsConstructor
@RequestMapping("/tools")
public class UploadApi {

    private final UploadService uploadService;

    /**
     * 上传文件至oss服务器
     */
    @ApiOperation("上传文件至oss服务器(旧：/file/upload)")
    @ApiImplicitParam(required = true, name = "file", value = "文件")
    @PostMapping("/file/upload/oss")
    public ResponseJson<Map<String, Object>> uploadFileToOss(MultipartFile file, @RequestHeader HttpHeaders headers) {
        return uploadService.uploadFileToOss(file, headers);
    }

    /**
     * 删除oss服务器的文件
     */
    @ApiOperation("删除oss服务器的文件(旧：/file/delete)")
    @ApiImplicitParam(required = true, name = "ossName", value = "OSS文件名")
    @PostMapping("/file/delete/oss")
    public ResponseJson<Void> deleteFileFromOss(String ossName) {
        return uploadService.deleteFileFromOss(ossName);
    }

    /**
     * FastDFS图片上传
     */
    @ApiOperation("图片上传(旧：/formData/MultiPictareaddData)")
    @ApiImplicitParam(required = true, name = "file", value = "图片")
    @PostMapping("/image/upload/multi")
    public ResponseJson<String> multiImageUpload(MultipartFile[] file, @RequestHeader HttpHeaders headers) {
        return uploadService.multiImageUpload(file, headers);
    }

    /**
     * 图片上传（ckeditor5富文本编辑器上传图片用，www,manager）
     */
    @ApiOperation("ckeditor5图片上传(旧：/formData/ckeditorImage)")
    @ApiImplicitParam(required = true, name = "upload", value = "图片")
    @PostMapping("/image/upload/ckeditor")
    public Map<String, Object> ckeditorImageUpload(MultipartFile upload, @RequestHeader HttpHeaders headers) {
        return uploadService.ckeditorImageUpload(upload, headers);
    }

}
