package com.caimei365.tools.mapper;

import com.caimei365.tools.model.bo.CouponDateBo;
import com.caimei365.tools.model.bo.HeheCouponBo;
import com.caimei365.tools.model.po.LogisticsInfoPo;
import com.caimei365.tools.model.po.SuperVipPo;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.Date;
import java.util.List;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/5/31
 */
@Mapper
public interface BaseMapper {
    /**
     * 近半年未签收物流列表
     */
    List<LogisticsInfoPo> getAllUnsignedLogistics();

    /**
     * 顺丰速运需要提供收/寄件人的电话号码查询
     */
    String getLogisticsPhoneByOrderId(Long orderId);

    /**
     * 更新数据库物流信息
     */
    void updateLogistics(LogisticsInfoPo logistics);

    List<SuperVipPo> findVip(Date startTime, Date endTime);

    String findMobile(Integer userId);
    /**
     * 获取时间到期且未下架的二手商品Id
     */
    List<Integer> getMaturitySecondProductIds(Date beforeDays);
    /**
     * 更新商品状态
     */
    void updateProductValidFlag(Integer productId, String validFlag);
    /**
     * 查询所有优惠券用户Id
     */
    List<Integer> getCouponUserIds();
    /**
     * 已领取优惠券
     */
    List<CouponDateBo> getReceivedCoupons(Integer userId);
    /**
     * 未领取优惠券
     */
    List<CouponDateBo> getUnclaimedCoupons(Integer userId);
    /**
     * 查询短链接是否存在
     */
    Integer findIdByShortLink(String shortLink);
    /**
     * 保存短链接信息
     *
     * @param markId    短信类型
     * @param shortLink 短链接
     * @param url       跳转地址
     */
    void insertShortLink(int markId, String shortLink, String url);
    /**
     * 保存短信发送条数+count
     *
     * @param markId 短信类型
     * @param count  增加条数
     */
    void updateSmsSendCount(int markId, int count);

    /**
     * 统计前一天商品访问量
     * @param productId 商品Id
     * @param views     访问量
     * @param viewTime 访问日期（天）
     */
    void insertProductViews(Integer productId, Integer views, Date viewTime);
}
