package com.caimei365.tools.service.impl;

import com.caimei365.tools.model.ResponseJson;
import com.caimei365.tools.service.LogisticsService;
import com.google.gson.Gson;
import com.kuaidi100.sdk.api.AutoNum;
import com.kuaidi100.sdk.api.QueryTrack;
import com.kuaidi100.sdk.core.IBaseClient;
import com.kuaidi100.sdk.pojo.HttpResult;
import com.kuaidi100.sdk.request.AutoNumReq;
import com.kuaidi100.sdk.request.QueryTrackParam;
import com.kuaidi100.sdk.request.QueryTrackReq;
import com.kuaidi100.sdk.response.QueryTrackResp;
import com.kuaidi100.sdk.utils.SignUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;


/**
 * 快递100API查询物流实现类
 *
 * @author : Charles
 * @date : 2021/5/18
 */
@Slf4j
@Service
public class LogisticsServiceImpl implements LogisticsService {
    /** 快递100授权KEY */
    private static final String LOGISTICS_KEY = "bEZlgVkW1438";
    /** 快递100企业版授权码 */
    private static final String LOGISTICS_CUSTOMER = "1414E0E533E3311FBA5E633C10F78306";

    /**
     * 智能获取物流信息
     *
     * @param number      快递单号
     * @param companyCode 快递公司的编码
     * @param mobile      收/寄件人的电话号码(顺丰必填)
     */
    @Override
    public ResponseJson<QueryTrackResp> getLogisticsByNumber(String number, String companyCode, String mobile) {
        if (StringUtils.isEmpty(number)) {
            return ResponseJson.error("参数错误：快递单号不能为空！", null);
        }
        // 智能获取物流公司代码
        AutoNumReq autoNumReq = new AutoNumReq();
        autoNumReq.setKey(LOGISTICS_KEY);
        autoNumReq.setNum(number);
        AutoNum baseClient = new AutoNum();
        try {
            String firstComByNum = baseClient.getFirstComByNum(autoNumReq);
            if (StringUtils.isEmpty(firstComByNum) && StringUtils.isNotEmpty(companyCode)) {
                firstComByNum = companyCode;
            }
            if (StringUtils.isNotEmpty(firstComByNum)) {
                // 顺丰需要提供手机号
                if ("shunfeng".equals(firstComByNum) && StringUtils.isEmpty(mobile)){
                    return ResponseJson.error(-2, "顺丰速运需要提供收/寄件人的电话号码查询，请提供后重试", null);
                }
                // 根据快递单号获取物流信息
                return queryLogistics(number, mobile, firstComByNum);
            }
            return ResponseJson.error("未能识别物流公司,请提供正确的快递单号后重试！", null);
        } catch (Exception e) {
            log.error("智能获取物流公司代码,错误信息：", e);
            return ResponseJson.error("智能获取物流公司出错,请提供物流公司代码后重试！", null);
        }
    }

    /**
     * 根据快递单号获取物流信息
     *
     * @param number      快递单号
     * @param companyCode 快递公司的编码
     * @param mobile      收/寄件人的电话号码(顺丰必填)
     */
    private ResponseJson<QueryTrackResp> queryLogistics(String number, String mobile, String companyCode) {
        // 设置参数信息
        QueryTrackParam queryTrackParam = new QueryTrackParam();
        queryTrackParam.setCom(companyCode);
        queryTrackParam.setNum(number);
        if (StringUtils.isNotEmpty(mobile)) {
            queryTrackParam.setPhone(mobile);
        }
        String param = new Gson().toJson(queryTrackParam);
        // 设置请求信息
        QueryTrackReq queryTrackReq = new QueryTrackReq();
        queryTrackReq.setParam(param);
        queryTrackReq.setCustomer(LOGISTICS_CUSTOMER);
        queryTrackReq.setSign(SignUtils.querySign(param , LOGISTICS_KEY, LOGISTICS_CUSTOMER));

        IBaseClient baseClient = new QueryTrack();
        try {
            HttpResult httpResult = baseClient.execute(queryTrackReq);
            if (httpResult.getStatus() == 200 && StringUtils.isNotBlank(httpResult.getBody())) {
                QueryTrackResp queryTrackResp = (new Gson()).fromJson(httpResult.getBody(), QueryTrackResp.class);
                log.info(">> 获取物流信息:"+number+" >> " + new Gson().toJson(queryTrackResp));
                return ResponseJson.success(queryTrackResp);
            }
            return ResponseJson.error(httpResult.toString(), null);
        } catch (Exception e) {
            log.error("获取物流信息,错误信息：", e);
            return ResponseJson.error("获取物流信息错误"+e.toString(), null);
        }
    }

}
