package com.caimei365.tools.service.impl;

import com.caimei365.tools.model.ResponseJson;
import com.caimei365.tools.service.UploadService;
import com.caimei365.tools.utils.FastDfsUtil;
import com.caimei365.tools.utils.OssUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/10/25
 */
@Slf4j
@Service
public class UploadServiceImpl implements UploadService {
    @Value("${caimei.imageDomain}")
    private String imageDomain;
    @Value("${spring.cloud.config.profile}")
    private String profile;
    @Resource
    private FastDfsUtil fastDfsUtil;

    /**
     * 上传文件至oss服务器
     *
     * @param file
     * @param headers
     */
    @Override
    public ResponseJson<Map<String, Object>> uploadFileToOss(MultipartFile file, HttpHeaders headers) {
        // 打印IP
        String ip = headers.getFirst("X-CLIENT-IP");
        log.info("【OSS文件上传】 X-CLIENT-IP : " + ip);
        Map<String, Object> map = new HashMap<>(3);
        String fileName = file.getOriginalFilename();
        String fileType = fileName.substring(fileName.lastIndexOf(".") + 1);
        String uuid = UUID.randomUUID().toString().replaceAll("-", "");
        String filePath = uuid + "." + fileType;
        String contentType = OssUtils.getContentType(fileName);
        try {
            //保存本地
            File localFile = OssUtils.ossUpload(file);
            String url = "";
            log.info("【OSS文件上传】>>>>>>>>>>>>>>>>默认路径：" + localFile.getAbsolutePath());
            //上传oss
            url = OssUtils.ossUpload(filePath, localFile, contentType);
            //删除本地文件
            OssUtils.deleteFile(localFile);
            map.put("fileName", fileName);
            map.put("ossName", filePath);
            map.put("url", url);
            log.info("【OSS文件上传】>>>>>>>>>>>>>>>>上传成功：" + url);
        } catch (IOException e) {
            log.error("【OSS文件上传】>>>>>>>>>>>>>>>>上传失败：" + e);
            return ResponseJson.error("上传文件失败!", null);
        }
        return ResponseJson.success(map);
    }

    /**
     * 删除oss服务器的文件
     *
     * @param ossName
     */
    @Override
    public ResponseJson<Void> deleteFileFromOss(String ossName) {
        if (StringUtils.isBlank(ossName)) {
            return ResponseJson.error("参数异常", null);
        }
        //删除oss服务器上的文件
        OssUtils.deleteSingleFile(ossName);
        return ResponseJson.success(null);
    }

    /**
     * FastDFS图片上传
     */
    @Override
    public ResponseJson<String> multiImageUpload(MultipartFile[] file, HttpHeaders headers){
        // 打印IP
        String ip = headers.getFirst("X-CLIENT-IP");
        log.info("【图片上传】 X-CLIENT-IP : " + ip);
        if (file != null && file.length > 0) {
            try {
                String fileUrl = null;
                for (MultipartFile fileItem : file) {
                    String url = saveFile(fileItem);
                    fileUrl = imageDomain + "/" + url;
                }
                log.info("【图片上传】>>>>>>>>>>>>>>>>上传成功:" + fileUrl);
                return ResponseJson.success("上传成功", fileUrl);
            } catch (IOException e) {
                log.error("【图片上传】>>>>>>>>>>>>>>>>上传失败:" + e);
                return ResponseJson.error("图片上传失败！",null);
            }
        } else {
            return ResponseJson.error("请传入图片！",null);
        }
    }

    /**
     * 图片上传（ckeditor5富文本编辑器上传图片用，www,manager）
     */
    @Override
    public Map<String, Object> ckeditorImageUpload(MultipartFile upload, HttpHeaders headers) {
        // 打印IP
        String ip = headers.getFirst("X-CLIENT-IP");
        log.info("【图片上传】 X-CLIENT-IP : " + ip);
        Map<String, Object> map = new HashMap<String, Object>();
        if (upload != null) {
            try {
                String url = saveFile(upload);
                String fileUrl = imageDomain + "/" + url;
                log.info("【图片上传】>>>>>>>>>>>>>>>>上传成功:" + fileUrl);
                map.put("uploaded", 1);
                map.put("url", fileUrl);
                map.put("msg", "上传成功");
            } catch (IOException e) {
                log.error("【图片上传】>>>>>>>>>>>>>>>>上传失败:" + e);
                map.put("msg", "上传失败");
            }
        } else {
            map.put("msg", "请传入图片!");
        }
        return map;
    }

    /**
     * 保存文件到FastDFS
     */
    private String saveFile(MultipartFile file) throws IOException {
        String originalFilename = file.getOriginalFilename();
        String randomStr = UUID.randomUUID().toString();
        assert originalFilename != null;
        int index = originalFilename.lastIndexOf(".");
        String name = originalFilename.substring(index);
        // 图片暂存本地服务器路径
        String filePath = "/mnt/newdatadrive/data/runtime/cloud-instance/server-tools/tempImage/";
        if ("dev".equals(profile)){
            filePath = "D:\\";
        }
        filePath += randomStr + name;
        // 临时图片
        File tempFile = new File(filePath);
        file.transferTo(tempFile);
        log.info("【图片上传】>>>>>>>>>>>>>>>>图片临时路径:" + filePath);
        String fileUrl = fastDfsUtil.uploadFile(filePath);
        // 删除临时图片
        boolean delete = tempFile.delete();
        log.info("【图片上传】>>>>>>>>>>>>>>>>删除临时图片:" + delete);
        return fileUrl;
    }
}
