package com.caimei365.tools.task;

import com.caimei365.tools.mapper.BaseMapper;
import com.caimei365.tools.model.bo.CouponDateBo;
import com.caimei365.tools.utils.CodeUtil;
import com.caimei365.tools.utils.DateUtils;
import com.caimei365.tools.utils.SmsUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 优惠券定时推送
 *
 * @author : Charles
 * @date : 2021/10/26
 */
@Slf4j
@Configuration
@EnableScheduling
@RequiredArgsConstructor
public class CouponTask {
    @Resource
    private BaseMapper baseMapper;
    @Value("${spring.cloud.config.profile}")
    private String profile;

    /**
     * 优惠券即将到期推送,定时中午12点推送
     */
    @Scheduled(cron = "0 0 12 * * ?")
    public void couponExpiring() {
        // 查询所有优惠券用户Id
        List<Integer> userIds = baseMapper.getCouponUserIds();
        int a = 0;
        int b = 0;
        int c = 0;
        Date date = new Date();
        //测试环境手机号允许发短信
        List<String> list = new ArrayList<>();
        list.add("15917362709");
        list.add("15814011616");
        list.add("15113936829");
        A:
        for (Integer userId : userIds) {
            String mobile = baseMapper.findMobile(userId);
            if (StringUtils.isBlank(mobile) || mobile.length() != 11) {
                log.info("机构手机号异常,userId>>>>>" + userId);
                continue;
            }
            // 正式环境或指定手机号
            if ("prod".equals(profile) || list.contains(mobile)) {
                // 已领取优惠券
                List<CouponDateBo> couponList = baseMapper.getReceivedCoupons(userId);
                String content = null;
                if (couponList != null && couponList.size() > 0) {
                    for (CouponDateBo coupon : couponList) {
                        if (DateUtils.differentDays(coupon.getStartDate(), coupon.getEndDate()) >= 15 && DateUtils.differentDays(date, coupon.getEndDate()) == 7) {
                            //7天后优惠券即将过期推送
                            String shortLink = getShortLink(8, 12, "https://www.caimei365.com/user/coupon-collection.html");
                            content = "您有优惠券即将过期，快戳采美网站链接www.caimei365.com/t/" + shortLink +
                                    " 或微信搜索“采美采购商城”小程序登录采美平台查看使用吧。关注公众号“采美365网”获取更多优惠和精彩资讯。退订回T";
                            String result = SmsUtil.sendSms(3, mobile, content);
                            a++;
                            continue A;
                        }
                        if (DateUtils.differentDays(date, coupon.getEndDate()) == 0) {
                            //当天优惠券即将过期推送
                            String shortLink = getShortLink(8, 13, "https://www.caimei365.com/user/coupon-collection.html");
                            content = "您有优惠券今日过期，快戳采美网站链接www.caimei365.com/t/" + shortLink +
                                    "  或微信搜索“采美采购商城”小程序登录采美平台查看使用吧。关注公众号“采美365网”获取更多优惠和精彩资讯。退订回T";
                            String result = SmsUtil.sendSms(3, mobile, content);
                            b++;
                            continue A;
                        }
                    }
                }
                // 未领取优惠券
                List<CouponDateBo> coupons = baseMapper.getUnclaimedCoupons(userId);
                if (couponList != null && couponList.size() > 0) {
                    for (CouponDateBo coupon : coupons) {
                        if (DateUtils.differentDays(date, coupon.getEndDate()) == 7) {
                            //7天后优惠券即将过期推送
                            String shortLink = getShortLink(8, 14, "https://www.caimei365.com/user/coupon-collection.html");
                            content = "您有优惠券尚未领取，优惠券即将失效，快戳采美网站链接www.caimei365.com/t/" + shortLink +
                                    " 或微信搜索“采美采购商城”小程序登录采美平台领取使用吧。关注公众号“采美365网”获取更多优惠和精彩资讯。退订回T";
                            String result = SmsUtil.sendSms(3, mobile, content);
                            c++;
                            continue A;
                        }
                    }
                }
            }
        }
        baseMapper.updateSmsSendCount(12, a);
        baseMapper.updateSmsSendCount(13, b);
        baseMapper.updateSmsSendCount(14, c);
    }

    /**
     * 生成短链接
     *
     * @param length 链接长度
     * @param markId 跳转类型
     * @param url    跳转地址
     */
    private String getShortLink(int length, int markId, String url) {
        String shortLink = CodeUtil.generateShortLink(length);
        Integer id = baseMapper.findIdByShortLink(shortLink);
        if (id != null && id > 0) {
            getShortLink(length, markId, url);
        }
        baseMapper.insertShortLink(markId, shortLink, url);
        return shortLink;
    }

}
